****************************************************************************
**
**  Filename: 
**            DoIntegrals.frm
**
**  Authors: 
**            Sven O. Moch and Peter Uwer
**
**  Purpose: 
**            Form script to calculate several loop integrals 
**            using the XSummer package.
**
**  Comments:
**
**
**  Revision: 
**            $Id: DoIntegrals.frm,v 1.0 2005/08/02 10:09:54 puwer Exp $
**
**            $Log: DoIntegrals.frm,v $
**            Revision 1.0  2005/08/02 10:09:54  puwer
**            Initial revision
**
**
****************************************************************************
*

#:	LargeSize   100000000
#:	SmallSize    20000000
#:	ScratchSize 100000000
#:	SortIOSize    2000000
#:	WorkSpace    10000000
#:	MaxTermSize   4000000


#define EXPANDEP "5"
#define TRUNCEP "2"
#define MAXSUM "2"
#define MAXWEIGHT "2"
#include declvars.h
#include declsums.h

#define PATH "tmp"

cf ln,Li;
CF I4q1q2, I4p1p4, I4p4q2, I4p1q1;
CF puretri,scaletri,Tri1,Tri3;
CF sEvac,bbox,ctopo,ctopo2spe;
Autodeclare CF hypergeom,appel;
CF Po,InvPo;


s [Integral];

s d;
Autodeclare S z, a, b, c, D;
PolyFun, acc;

autodeclare s nu;
s s123;

Off Statistics;
.global

L integral = [Integral];

*
* Chose an integral from the list:
*
#call SelectIntegral(`TYPE',`INTEGRAL')

*
*	Insert representation in terms of nested sums for the integrals:
*
#call Int2Sum(`TYPE')

*
* Expand the Gamma-functions
*
#call Simplify

*
* To avoid generating to much terms and thus slowing down everything we
* expand the prefactor of the C-topology separately and multiply after the
* expansion. We have to make sure that the expansion is done sufficiently high
* so that we get the desired order in the epsilon expansion. In principle
* a clever procedure may speed up the process.
*
#if `TYPE' == "CTOPO"
	.sort;
	drop Ctopoprefac;
	multiply,Ctopoprefac;
	repeat id acc(x1?)*acc(x2?) = acc(x1*x2);
	.sort;
#endif

*
* Prepare the epsilon expansion of expressions of the form x^(n +/- ep)
*
SplitArg,((ep)),pow;
#do iii = 1,5
	id pow(x1?,x2?,+`iii'*ep) = pow(x1,x2)*pow(x1,+`iii'*ep);
	id pow(x1?,x2?,-`iii'*ep) = pow(x1,x2)*pow(x1,-`iii'*ep);
#enddo
id pow(x?,n0?pos0_) = x^n0;
id pow(x?,n0?neg_) = den(x)^(-n0);

id acc(x?) = x;

*
* Do the epsilon expansion of expressions containing powers of epsilon
* 
#do iii = 1,5
	id pow(x1?,+`iii'*ep)*ep^a? = ep^a*
		sum_(jjj,0,`EXPANDEP'-a,
						(-`iii'*ep)^jjj*invfac_(jjj)*S(R(1),X(1-x1),inf)^jjj);
	id pow(x1?,-`iii'*ep)*ep^a? = ep^a*
		sum_(jjj,0,`EXPANDEP'-a,
						(`iii'*ep)^jjj*invfac_(jjj)*S(R(1),X(1-x1),inf)^jjj);
#enddo

*
* Truncate the series in epsilon
*
id ep^n0?{,>`TRUNCEP'} = 0;
#switch `TYPE'
	#case CTOPO
		id ep^n0?{,`TRUNCEP'} = 0;
	#break
#endswitch
.sort(PolyFun=acc): trunc eps;
id ep^n?*acc(x?) = acc(ep^n*x);
#do iii = 2,6;
	id z`iii'^n?*acc(x?) = acc(z`iii'^n*x);
#enddo

*
* Call the routine for summation 
*
#call DoSum(2,1)

*
* Final clean up  argument of S-functions
*
Argument S;
	Argument X;
		repeat;
			id den(1 + x1^(-1)*x2) = x1*den(x1 + x2);
			id den(1 + x2^(-1)*x1) = x2*den(x1 + x2);
			id x2*den(x1 + x2) = 1 - x1*den(x1 + x2);
		endrepeat;
	EndArgument;
EndArgument;

*
* Cleaning up the pow-function
*
id  pow(x1?,0) = 1;
id  pow(1,x1?) = 1;
id  pow(x?,n0?pos0_) = x^n0;
id  pow(x?,n0?neg_) = den(x)^(-n0);

*
* Evaluate the S sums as far as possible
*
#call SubsS(1)

.sort;


*
* Some customized relations
* 
repeat id den(x?int_) = 1/x;
repeat id den(den(x1?)) = x1;


*
* Convert to Li notation
*
*
* First convert the S-sums to Z-sums
*
#call ConvStoZ(S,Z)

*
* and then convert the Z-sums to the generalized polylogarithms
*
id Z(R(?a),X(?x),inf) = Li(R(reverse_(?a)),X(reverse_(?x)));

*
* Simplify arguments of the generalized polylogarithms
*
Argument Li;
	Argument X;
		repeat,id x2*den(x1 + x2) = 1 - x1*den(x1 + x2);
	EndArgument;
EndArgument;


id acc(x?) = x;

*
* Try to generate `normal form' for the rational functions in x1,x2
*	We just do partional fractioning.
*
#call CustomizeDen
#call CustomizeDen
.sort: CustomizeDen;

#call ParFrac(1)
.sort: ParFrac-1;

#call ParFrac(2)
.sort: ParFrac-2;

*
* 	Emergency brake: At this point there shouldn't be any sum or 
* 	                 related object left. 
*
if ( count(S,1,pow,1,S1,1,j1,1,pow1,1,den1,1,fac1,1,invfac1,1) );
	Print "Unresolved sums in %t";
	setexitflag;
endif;

.sort
PolyFun;
id acc(x?) = x;
id Li(R(?a),X(?y)) = Li(?a,?y);


id ctopo(-4711) = 1;

.sort;

*
* Print the result on the screen:
*
format nospaces;
format,maple,120;
print +s;

.sort;

*
* Write out the result in form of an `id' statement so that we can use
* it later in another form script. 
*
L help = [Integral];

#call SelectIntegral(`TYPE',`INTEGRAL')

#switch `TYPE'
	#case CTOPO
    id ctopo(x1?$arg1,x2?$arg2,x3?$arg3,x4?$arg4,x5?$arg5,x6?$arg6) = 0;
    .sort;
		#append <`PATH'/ctoponf`INTEGRAL'>
		#write <`PATH'/ctoponf`INTEGRAL'> "id ctoponf(`$arg1',`$arg2',`$arg3',`$arg4',`$arg5',`$arg6',x1?,x2?)  ="
		#write <`PATH'/ctoponf`INTEGRAL'> "      %e",integral
	#break
	#case BBOX			
		id bbox(x1?$arg1,x2?$arg2,x3?$arg3,x4?$arg4,x5?$arg5,?a) = 0;
		.sort;
		#append <`PATH'/bbox`INTEGRAL'>
		#write <`PATH'/bbox`INTEGRAL'> "id bbox(`$arg1',`$arg2',`$arg3',`$arg4',`$arg5',x1?,x2?)  ="
		#write <`PATH'/bbox`INTEGRAL'> "      %e",integral
	#break
	#case TRIZERO			
		id Tri1(?a$args) = 0;	
		.sort;
		#append <`PATH'/Trizero>
		#write <`PATH'/Trizero> "id Tri1(`$args')  ="
		#write <`PATH'/Trizero> "      %e",integral
	#break
	#case TRIMASS
		id Tri3(?a$args) = 0;
		.sort
		#append <`PATH'/Trimass`INTEGRAL'>
		#write <`PATH'/Trimass`INTEGRAL'> "id Tri3(`$args',x1?)  ="
		#write <`PATH'/Trimass`INTEGRAL'> "      %e",integral
	#break
#endswitch
print +s, help;
.end
*************************************************************************
* Local variables:
* modify-insert-tag: nil
* comment-start: "*"
* tab-width: 2
* End:
*************************************************************************
